@@ -118,7 +118,26 @@ expInOut = function(t) {
   ifelse(t * 2 <= 1, 2^(-10 * (1-2*t))/2, (2 - 2^(-10 * (2*t-1)))/2)
 }
 
-#' Cubic-in-out
+#' Slerp
+#' 
+#' @param vec1 Value
+#' @param vec2 Value
+#' @param n Value
+#' @return number
+#'
+#' @keywords internal
+slerp = function(vec1, vec2, n) {
+  t = seq(0,1,length.out = n+2)
+  subtended_angle = acos(sum(vec1*vec2))
+  return_vecs = list()
+  for(i in 1:(n+2)) {
+    return_vecs[[i]] = sin((1-t[i])*subtended_angle)/sin(subtended_angle) * vec1 + 
+      sin(t[i]*subtended_angle)/sin(subtended_angle) * vec2
+  }
+  return(return_vecs)
+}
+
+#' Tween
 #' 
 #' @param vals Numeric values
 #' @param n Frames
@@ -127,6 +146,9 @@ expInOut = function(t) {
 #'
 #' @keywords internal
 tween = function(vals, n, ease = "cubic") {
+  if(length(vals) == 1) {
+    return(rep(vals,n))
+  }
   len_vals = rep(0, length(vals)-1)
   free_vals = n - length(vals)
   counter = 1
@@ -284,3 +306,97 @@ RotateAxis = function(theta, axis) {
   m[3,4] = 0
   return(m)
 }
+
+#' Calculate Final Angle
+#'
+#' @keywords internal
+calculate_final_twist = function(full_control_points, 
+                                 breaks, t_vals, 
+                                 t_vec, s_vec, r_vec) {
+  r_vec0 = r_vec
+  s_vec0 = s_vec
+  t_vec0 = t_vec
+  for(i in seq_len(breaks-1)) {
+    t_val0 = t_vals[i]
+    if(t_val0 < 0) {
+      t_val0 = 0
+    }
+    t_val1 = t_vals[i+1]
+    if(t_val1 < 0) {
+      t_val1 = 0
+    }
+    
+    rot_mat = matrix(c(s_vec,r_vec,t_vec),3,3)
+    t_temp0 = t_val0-floor(t_val0)
+    if(i != breaks-1) {
+      t_temp1 = t_val1-floor(t_val1)
+    } else {
+      t_temp1 = 1
+    }
+    
+    i0 = floor(t_val0) + 1
+    if(i != breaks-1) {
+      i1 = floor(t_val1) + 1
+    } else {
+      i1 = max(c(1,floor(t_val1+1e-8)))
+    }
+    
+    
+    cp0 = full_control_points[[i0]]
+    if(i1 <= length(full_control_points)) {
+      cp1 = full_control_points[[i1]]
+    } else {
+      cp1 = cp0
+    }
+    
+    x0 = eval_bezier(cp0,t_temp0)
+    x1 = eval_bezier(cp1,t_temp1)
+    
+    #Evaluate next set of vectors
+    v1 = x1-x0
+    c1 = sum(v1*v1)
+    rl = r_vec - (2 / c1) * sum(v1*r_vec) * v1
+    tl = t_vec - (2 / c1) * sum(v1*t_vec) * v1
+    
+    next_deriv = eval_bezier_deriv(cp1,t_temp1)
+    t_vec_prev = next_deriv/sqrt(sum(next_deriv*next_deriv))
+    
+    v2 = t_vec_prev - tl
+    c2 = sum(v2*v2)
+    if(c2 != 0) {
+      t_vec = t_vec_prev
+      r_vec = rl - (2 / c2) * sum(v2*rl) * v2
+      s_vec = cross_prod(t_vec,r_vec)
+    }
+  }
+  angle_r = acos(sum(r_vec0*r_vec))
+  angle_s = acos(sum(s_vec0*s_vec))
+  angle_t = acos(sum(t_vec0*t_vec))
+  
+  return(c(angle_r,angle_s,angle_t))
+}
+
+#' Add Points to Polygon
+#' 
+#' @param polygon Polygon
+#' @param added_points Default `0`
+#' @return matrix
+#'
+#' @keywords internal
+add_points_polygon = function(polygon, added_points = 0L) {
+  existing_verts = nrow(polygon)
+  total_verts = existing_verts + (existing_verts-1L) * added_points
+  return_polygon = matrix(0, ncol=3,nrow=total_verts)
+  return_polygon[,1] = tween(polygon[,1], n = total_verts, ease = "linear")
+  return_polygon[,2] = tween(polygon[,2], n = total_verts, ease = "linear")
+  return(return_polygon)
+}
+
+#' Run Documentation
+#' 
+#' @return bool
+#'
+#' @keywords internal
+run_documentation = function() {
+  return(identical(Sys.getenv("IN_PKGDOWN"), "true"))
+}
