@@ -137,7 +137,7 @@ std::shared_ptr<hitable> build_scene(IntegerVector& type,
     if(has_bump(i)) {
       bump[i] = std::make_shared<bump_texture>(bump_textures[i], 
                                                nvecb[i][0], nvecb[i][1], nvecb[i][2], 
-                              bump_intensity(i));
+                                               bump_intensity(i),temp_repeat[0], temp_repeat[1]);
     }
     if(has_roughness(i)) {
       roughness[i] = std::make_shared<roughness_texture>(roughness_textures[i], 
@@ -154,8 +154,9 @@ std::shared_ptr<hitable> build_scene(IntegerVector& type,
     } else if (type(i) == 9) {
       prop_len = 6;
     }
-    if(is_shared_mat(i) && shared_materials->size() > static_cast<size_t>(shared_id_mat(i))) {
-      tex = shared_materials->at(shared_id_mat(i));
+
+    if(is_shared_mat(i) && shared_materials->size() > static_cast<int>(shared_id_mat(i)-1)) {
+      tex = shared_materials->at(shared_id_mat(i)-1);
     } else {
       if(type(i) == 1) {
         if(isimage(i)) {
@@ -169,7 +170,7 @@ std::shared_ptr<hitable> build_scene(IntegerVector& type,
           tex = std::make_shared<lambertian>(std::make_shared<checker_texture>(
             std::make_shared<constant_texture>(point3f(tempchecker(0),tempchecker(1),tempchecker(2))),
             std::make_shared<constant_texture>(point3f(tempvector(0),tempvector(1),tempvector(2))),
-                                                   tempchecker(3)));
+                                             tempchecker(3)));
         } else if (isgradient(i) && !is_world_gradient(i)) {
           tex = std::make_shared<lambertian>(std::make_shared<gradient_texture>(point3f(tempvector(0),tempvector(1),tempvector(2)),
                                                                                 point3f(tempgradient(0),tempgradient(1),tempgradient(2)),
@@ -454,7 +455,7 @@ std::shared_ptr<hitable> build_scene(IntegerVector& type,
         }
       }
     }
-    if(is_shared_mat(i) && shared_materials->size() <= static_cast<size_t>(shared_id_mat(i)) ) {
+    if(is_shared_mat(i) && shared_materials->size() < static_cast<size_t>(shared_id_mat(i)) ) {
       shared_materials->push_back(tex);
     }
     //Generate center vector
@@ -752,7 +753,9 @@ std::shared_ptr<hitable> build_scene(IntegerVector& type,
       List mesh_entry = mesh_list(i);
       std::shared_ptr<hitable> entry = std::make_shared<mesh3d>(mesh_entry, tex,
                                   shutteropen, shutterclose, bvh_type, rng,
-                                  ObjToWorld,WorldToObj, isflipped(i));
+                                  ObjToWorld,WorldToObj, isflipped(i),
+                                  prop_len, tempvector, temp_glossy, sigma(i),
+                                  lightintensity(i));
       if(has_animation(i)) {
         entry = std::make_shared<AnimatedHitable>(entry, Animate);
       }
@@ -1028,9 +1031,12 @@ std::shared_ptr<hitable> build_imp_sample(IntegerVector& type,
     return(entry);
   } else {
     List mesh_entry = mesh_list(i);
+    NumericVector temp_glossy(9);
     std::shared_ptr<hitable> entry = std::make_shared<mesh3d>(mesh_entry, tex,
                                 shutteropen, shutterclose, bvh_type, rng, 
-                                ObjToWorld,WorldToObj, false);
+                                ObjToWorld, WorldToObj, false,
+                                prop_len, tempvector, temp_glossy,1.0f,
+                                0);
     if(has_animation(i)) {
       entry = std::make_shared<AnimatedHitable>(entry, Animate);
     }
