@@ -37,11 +37,14 @@ class bvh_node : public hitable {
 
     virtual bool bounding_box(Float t0, Float t1, aabb& box) const;
     
-    Float pdf_value(const vec3& o, const vec3& v, random_gen& rng, Float time = 0);
-    Float pdf_value(const vec3& o, const vec3& v, Sampler* sampler, Float time = 0);
-    vec3 random(const vec3& o, random_gen& rng, Float time = 0);
-    vec3 random(const vec3& o, Sampler* sampler, Float time = 0);
+    Float pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time = 0);
+    Float pdf_value(const point3f& o, const vec3f& v, Sampler* sampler, Float time = 0);
+    vec3f random(const point3f& o, random_gen& rng, Float time = 0);
+    vec3f random(const point3f& o, Sampler* sampler, Float time = 0);
     
+    std::string GetName() const {
+      return(std::string("BVH Node"));
+    }
     std::shared_ptr<hitable> left;
     std::shared_ptr<hitable> right;
     aabb box;
