@@ -10,7 +10,7 @@
 
 class ImplicitShape { 
   public: 
-    virtual float getDistance(const point3f& from) const = 0; 
+    virtual Float getDistance(const point3f& from) const = 0; 
     virtual bool bbox(Float t0, Float t1, aabb& box) const = 0; 
     virtual ~ImplicitShape() {} 
 }; 
@@ -19,7 +19,7 @@ class csg_sphere : public ImplicitShape {
   public: 
     csg_sphere(const vec3f& c, const float& r) : 
       center(c), radius(r) {} 
-    float getDistance(const point3f& from) const { 
+    Float getDistance(const point3f& from) const { 
       return((from - center).length() - radius); 
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -37,7 +37,7 @@ class csg_plane : public ImplicitShape {
       axis.build_from_w(n);
       axis.swap_yz();
     } 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       Float dist   = dot(axis.v(),from - pointOnPlane);
       Float dist_x = dot(axis.u(),from - pointOnPlane);
       Float dist_z = dot(axis.w(),from - pointOnPlane);
@@ -79,7 +79,7 @@ class csg_box : public ImplicitShape {
 class csg_rounded_box : public ImplicitShape { 
   public: 
     csg_rounded_box(const vec3f& c, vec3f width_, Float radius) :  center(c), width(width_), radius(radius){} 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from = from_old  - center;
       vec3f q = Abs(from)  - width/2;
       const static vec3f zeros(0,0,0);
@@ -98,7 +98,7 @@ class csg_rounded_box : public ImplicitShape {
 class csg_list : public ImplicitShape { 
   public: 
     csg_list(std::vector<std::shared_ptr<ImplicitShape> > shapes) : shapes(shapes) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       float min_dist = INFINITY;
       float temp;
       for(const auto& shape: shapes) {
@@ -124,7 +124,7 @@ class csg_torus : public ImplicitShape {
   public: 
     csg_torus(const point3f& c, float ring_radius, float cross_radius) : center(c), 
       ring_radius(ring_radius), cross_radius(cross_radius) {} 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from = from_old - center;
       vec2f q = vec2f(std::sqrt(from.x()*from.x() + from.z()*from.z()) - ring_radius, from.y());
       return(q.length()-cross_radius);
@@ -143,7 +143,7 @@ class csg_torus : public ImplicitShape {
 class csg_capsule : public ImplicitShape { 
   public: 
     csg_capsule(point3f start, point3f end, Float radius) :  start(start), end(end), radius(radius){} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       vec3f pa = from - start; 
       vec3f ba = end - start;
       float h = clamp( dot(pa,ba)/dot(ba,ba), 0.0, 1.0 );
@@ -167,7 +167,7 @@ class csg_cylinder : public ImplicitShape {
       baba = dot(ba,ba);
       inv_baba = 1.0/baba;
     } 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       vec3f pa = from - start; 
       float paba = dot(pa,ba);
       float x = (pa*baba-ba*paba).length() - radius*baba;
@@ -193,7 +193,7 @@ class csg_ellipsoid : public ImplicitShape {
     csg_ellipsoid(const point3f& c, point3f axes) : center(c), axes(axes)  {
       inv_axes = point3f(1.0/axes.x(),1.0/axes.y(),1.0/axes.z());
     } 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       point3f from = point3f(from_old - center); 
       float k0 = (from * inv_axes).length();
       float k1 = (from * (inv_axes*inv_axes)).length();
@@ -210,7 +210,7 @@ class csg_rounded_cone : public ImplicitShape {
   public: 
     csg_rounded_cone(point3f start, point3f end, Float r1, Float r2) :  
       start(start), end(end), r1(r1), r2(r2) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       vec3f pa = from - start; 
       vec3f ba = end - start;
       float l2 = dot(ba,ba);
@@ -251,7 +251,7 @@ class csg_cone : public ImplicitShape {
       axis.build_from_w(end - start);
       axis.swap_yz();
     } 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from_trans = axis.world_to_local(from_old - start);
       vec3f from = from_trans - vec3f(0,height,0);
       vec2f q = vec2f(radius,-height);
@@ -283,7 +283,7 @@ class csg_pyramid : public ImplicitShape {
       m2_inv = 1/m2;
       m2_inv_buff = 1 / (m2+0.25);
     } 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from = from_old - center_bottom;
       from = from * base_inv;
       from.e[0] = std::fabs(from.e[0]); 
@@ -320,7 +320,7 @@ class csg_triangle : public ImplicitShape {
       ac = a - c; 
       nor = cross( ba, ac );
     } 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       vec3f pa = from - a;
       vec3f pb = from - b;
       vec3f pc = from - c;
@@ -352,7 +352,7 @@ class csg_elongate : public ImplicitShape {
   public: 
     csg_elongate(std::shared_ptr<ImplicitShape> shape, point3f center,vec3f elongate) : 
       shape(shape),center(center), elongate(elongate) {} 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from = from_old - center;
       point3f q = from - clamp(from, -elongate, elongate);
       return(shape->getDistance(q + center));
@@ -371,7 +371,7 @@ class csg_elongate_robust : public ImplicitShape {
   public: 
     csg_elongate_robust(std::shared_ptr<ImplicitShape> shape, point3f center, vec3f elongate) : 
       shape(shape),center(center),elongate(elongate) {} 
-    float getDistance(const point3f& from_old) const {
+    Float getDistance(const point3f& from_old) const {
       vec3f from = from_old - center;
       const static vec3f zeros(0,0,0);
       const static vec3f inf(INFINITY,INFINITY,INFINITY);
@@ -393,7 +393,7 @@ class csg_round : public ImplicitShape {
   public: 
     csg_round(std::shared_ptr<ImplicitShape> shape, Float r) : 
       shape(shape),r(r) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       return(shape->getDistance(from) - r);
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -409,7 +409,7 @@ class csg_onion : public ImplicitShape {
   public: 
     csg_onion(std::shared_ptr<ImplicitShape> shape, Float thickness) : 
     shape(shape),thickness(thickness) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       return(fabs(shape->getDistance(from)) - thickness);
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -425,7 +425,7 @@ class csg_scale : public ImplicitShape {
   public: 
     csg_scale(std::shared_ptr<ImplicitShape> shape, Float scale) : 
       shape(shape),scale(scale) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       return(shape->getDistance(from/scale)*scale);
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -500,7 +500,7 @@ class csg_rotate : public ImplicitShape {
       aabb new_box(temp_min,temp_max);
       box_cache = new_box;
     } 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       return(shape->getDistance(axis.world_to_local(from - pivot_point) + vec3f(pivot_point)));
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -518,7 +518,7 @@ class csg_translate : public ImplicitShape {
   public: 
     csg_translate(std::shared_ptr<ImplicitShape> shape, vec3f translate) : 
       shape(shape),translate(translate) {} 
-    float getDistance(const point3f& from) const {
+    Float getDistance(const point3f& from) const {
       return(shape->getDistance(from - translate));
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
@@ -576,7 +576,7 @@ class CSG : public ImplicitShape {
       const std::shared_ptr<ImplicitShape> s1, 
       const std::shared_ptr<ImplicitShape> s2, 
       Args&& ... args) : op(std::forward<Args>(args) ...), shape1(s1), shape2(s2) {} 
-    float getDistance(const point3f& from) const { 
+    Float getDistance(const point3f& from) const { 
       return(op(shape1->getDistance(from), shape2->getDistance(from))); 
     } 
     virtual bool bbox(Float t0, Float t1, aabb& box) const {
