@@ -31,7 +31,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
       p1 *= radius / p1.length(); 
       vec3f normal = (p1 - center) / radius;
       get_sphere_uv(normal, u, v);
-      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+      if(alpha_mask->value(u, v, rec.p) < rng.unif_rand()) {
         is_hit = false;
       }
     }
@@ -40,7 +40,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random
       p2 *= radius / p2.length(); 
       vec3f normal = (p2 - center) / radius;
       get_sphere_uv(normal, u, v);
-      if(alpha_mask->value(u, v, rec.p).x() < rng.unif_rand()) {
+      if(alpha_mask->value(u, v, rec.p) < rng.unif_rand()) {
         if(!is_hit) {
           return(false);
         }
@@ -152,7 +152,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
       p1 *= radius / p1.length(); 
       vec3f normal = (p1 - center) / radius;
       get_sphere_uv(normal, u, v);
-      if(alpha_mask->value(u, v, rec.p).x() < sampler->Get1D()) {
+      if(alpha_mask->value(u, v, rec.p) < sampler->Get1D()) {
         is_hit = false;
       }
     }
@@ -161,7 +161,7 @@ bool sphere::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sample
       p2 *= radius / p2.length(); 
       vec3f normal = (p2 - center) / radius;
       get_sphere_uv(normal, u, v);
-      if(alpha_mask->value(u, v, rec.p).x() < sampler->Get1D()) {
+      if(alpha_mask->value(u, v, rec.p) < sampler->Get1D()) {
         if(!is_hit) {
           alpha_miss = true;
         }
